/*
 * include/platform/wismo.h
 *
 * This file contains the hardware specific definitions for GigaSysNet Wismo
 *
 * $Id: wismo.h,v 1.1 2002/11/19 10:50:54 cgjeong Exp $
 *
 */

#include "config.h"
#include "sa1100.h"

/*
 * Memory map
 */
#include "sizes.h"

#define ROM_BASE0		0x00000000      /* base address of rom bank 0 */
#define ROM_BASE1		0x02000000      /* base address of rom bank 1 */
#define DRAM_BASE0		0xC0000000      /* base address of dram bank 0 */
#define DRAM_BASE1		0xC8000000	/* base address of dram bank 1 */

/* Global definitions */
#define FLASH_BASE		ROM_BASE0
#define FLASH_SIZE		SZ_16M
#define FLASH_UNCACHED_BASE	0x04000000	/* to mapping flash memory */
#define FLASH_BUSWIDTH		2

#define DRAM_BASE		DRAM_BASE0
#define DRAM_SIZE		SZ_64M

#define MTD_PART_SIZE		SZ_16K
#define MTD_PART_OFFSET		0x00000000
#define PARAMETER_TLB_SIZE	SZ_16K
#define PARAMETER_TLB_OFFSET	0x00004000
#define LINUX_CMD_SIZE		SZ_16K
#define LINUX_CMD_OFFSET	0x00008000
#define VIVI_PRIV_SIZE		(MTD_PART_SIZE + PARAMETER_TLB_SIZE + LINUX_CMD_SIZE)

/* ROM */
#define VIVI_ROM_BASE		0x00000000
#define VIVI_PRIV_ROM_BASE	0x00FC0000

/* RAM */
#define VIVI_RAM_SIZE		SZ_1M
#define VIVI_RAM_BASE		(DRAM_BASE + DRAM_SIZE - VIVI_RAM_SIZE)
#define HEAP_SIZE		SZ_1M
#define HEAP_BASE		(VIVI_RAM_BASE - HEAP_SIZE)
#define MMU_TABLE_SIZE		SZ_16K
#define MMU_TABLE_BASE		(HEAP_BASE - MMU_TABLE_SIZE)
#define VIVI_PRIV_RAM_BASE	(MMU_TABLE_BASE - VIVI_PRIV_SIZE)
#define STACK_SIZE		SZ_32K
#define STACK_BASE		(VIVI_PRIV_RAM_BASE - STACK_SIZE)
#define RAM_SIZE		(STACK_BASE - DRAM_BASE)
#define RAM_BASE		DRAM_BASE

/*
 * Architecture magic and machine type
 */
#include "architecture.h"
#define MACH_TYPE		249
#define ARCHITECTURE_MAGIC	((ARM_PLATFORM << 24) | (ARM_SA1100_CPU << 16) | \
				  MACH_TYPE)

/*
 * CPU register settings
 */
#define DRAM_MDCNFG_ENABLE	MDCNFG_BANK0_ENABLE
#define DRAM_MDCNFG_64MB	(DRAM_MDCNFG_ENABLE | MDCNFG_DTIM0_SDRAM | \
				MDCNFG_DWID0_32B | MDCNFG_DRAC0(6) | \
				MDCNFG_TRP0(2) | MDCNFG_TDL0(3) | MDCNFG_TWR0(1))
#define DRAM_MDCNFG		DRAM_MDCNFG_64MB
#define DRAM_CAS0_WAVEFORM0	0xAAAAAA9F
#define DRAM_CAS0_WAVEFORM1	0xAAAAAAAA
#define DRAM_CAS0_WAVEFORM2	0xAAAAAAAA
#define DRAM_MDREFR		(MDREFR_TRASR(7) | MDREFR_DRI(100) | \
				 MDREFR_E1PIN | MDREFR_K1RUN)
#define MSC0_CONFIG	\
                ( ((MSC_RT_ROMFLASH | MSC_RBW16 | MSC_RDF(31) | \
                    MSC_RDN(31) | MSC_RRR(7)) << 0 )  /* Bank 0 flash */ | \
                  ((MSC_RT_ROMFLASH | MSC_RBW16 | MSC_RDF(31) | \
                    MSC_RDN(31) | MSC_RRR(7)) << 16 )  /* Bank 1 flash */ \
                )
#define MSC1_CONFIG \
                ( ((MSC_RT_SRAM_012 | MSC_RBW16 | MSC_RDF(31) | MSC_RDN(31) | \
                    MSC_RRR(7)) << 0 )  /* Bank 2 KEYPAD */ | \
                  ((MSC_RT_ROMFLASH | MSC_RBW32 | MSC_RDF(31) | MSC_RDN(31) | \
                    MSC_RRR(7)) << 16 )  /* Bank 3 Power Ctrl Registers */ \
                )
#define MSC2_CONFIG \
                ( ((MSC_RT_SRAM_012 | MSC_RBW16 | MSC_RDF(31) | MSC_RDN(31) | \
                    MSC_RRR(7)) << 0 )  /* Bank 4 MIDI */ | \
                  ((MSC_RT_SRAM_012 | MSC_RBW16 | MSC_RDF(31) | MSC_RDN(31) | \
                    MSC_RRR(7)) << 16 )  /* Bank 5 MEDIAQ */ \
                )
#define MECR_CONFIG	0x994a994a

/*
 * Misc.
 */
#define UART_BAUD_RATE		115200

/*
 * GPIO/EGPIO settings
 */

/* Power Control Register */
#define WISMO_PCR_BASE	0x18000000

#define WISMO_PCR	__REG(WISMO_PCR_BASE)

#define WISMO_PCR_INIT	(0x14071407) 
